"""
Module: 'network' on micropython-v1.25.0-rp2-RPI_PICO2_W
"""

# MCU: {'build': '', 'ver': '1.25.0', 'version': '1.25.0', 'port': 'rp2', 'board': 'RPI_PICO2_W', 'mpy': 'v6.3', 'family': 'micropython', 'cpu': 'RP2350', 'arch': 'armv7emsp'}
# Stubber: v1.24.0
from __future__ import annotations
from typing import Final
from _typeshed import Incomplete

STA_IF: Final[int] = 0
STAT_IDLE: Final[int] = 0
STAT_NO_AP_FOUND: Final[int] = -2
STAT_WRONG_PASSWORD: Final[int] = -3
STAT_GOT_IP: Final[int] = 3
AP_IF: Final[int] = 1
STAT_CONNECTING: Final[int] = 1
STAT_CONNECT_FAIL: Final[int] = -1

def hostname(*args, **kwargs) -> Incomplete: ...
def ipconfig(*args, **kwargs) -> Incomplete: ...
def route(*args, **kwargs) -> Incomplete: ...
def country(*args, **kwargs) -> Incomplete: ...

class WLAN:
    PM_POWERSAVE: Final[int] = 17
    SEC_OPEN: Final[int] = 0
    SEC_WPA2_WPA3: Final[int] = 20971524
    SEC_WPA3: Final[int] = 16777220
    SEC_WPA_WPA2: Final[int] = 4194310
    PM_PERFORMANCE: Final[int] = 10555714
    IF_AP: Final[int] = 1
    IF_STA: Final[int] = 0
    PM_NONE: Final[int] = 16
    def ipconfig(self, *args, **kwargs) -> Incomplete: ...
    def status(self, *args, **kwargs) -> Incomplete: ...
    def ioctl(self, *args, **kwargs) -> Incomplete: ...
    def send_ethernet(self, *args, **kwargs) -> Incomplete: ...
    def isconnected(self, *args, **kwargs) -> Incomplete: ...
    def scan(self, *args, **kwargs) -> Incomplete: ...
    def config(self, *args, **kwargs) -> Incomplete: ...
    def ifconfig(self, *args, **kwargs) -> Incomplete: ...
    def active(self, *args, **kwargs) -> Incomplete: ...
    def disconnect(self, *args, **kwargs) -> Incomplete: ...
    def connect(self, *args, **kwargs) -> Incomplete: ...
    def deinit(self, *args, **kwargs) -> Incomplete: ...
    def __init__(self, *argv, **kwargs) -> None: ...
